//GTT Control Panel Example demo
//Arduino Uno with Matrix Orbital GTT70A
//Created By Daniel Divino, September 19th, 2016
//support@matrixorbital.ca
//www.matrixorbital.ca/appnotes
//www.lcdforums.ca

#include <Wire.h>
#include <stdlib.h>

//global variables
#define gttAddress 0x28 //Define default 8bit I2C address of 0x50 >> 1 for 7bit Arduino

int oldValue = 0;
int region;

void setup() {
  Wire.begin();
  Serial.begin(9600);
  delay(500);
  Serial.println("Screen Tracking Ready");

}

void loop() {  
  int data =i2cRead();
  switch (data){
    case 252:{
      data = i2cRead();
      switch(data){
        case 135:{
          int event = ReadEvent();  //Read the touch input event
          region = ReadRegion();  //Read the region that was just pressed          
          if ((event == 2 && region == 2) || (event == 2 && region == 3)){
            requestSliderValue(region); //Get the slider value when a slider is pressed
          }                    
          if (region == 5 || region == 6){
           requestToggleState(region); //Get the Toggle state when a toggle is pressed
          }
        }
        break;
        case 167:{
          int value = readSliderValue();  //Read the Slider value when a slider is pressed
          updateGTT(value); //Update the labels on the GTT with the new selected values
        }
        break;
      }
    }
    break;
    default:{      
    }    
    break;  
  }
}

//GTT Communication Functions
void updateGTT(int value){
  if (value != oldValue){
    oldValue = value;                         
    String valueString = String(value,DEC);
    String fulltext = valueString + "PSI" ;           
    char* labelUpdate = (char*)fulltext.c_str();    
    updateLabel(region,0, labelUpdate,fulltext.length());
    updateBargraphValue(region, value);
  }  
}

//GTT commands
int requestSliderValue(int region){
  char command[] = {254, 167, region};  //Send the Slider value command parameters
  i2cWrite(command, sizeof(command));
}

int readSliderValue(){
  int returnLength = i2cRead() << 16 | i2cRead();  //Read message length  
  int result = i2cRead(); //Read the eStatus result of the command
  int value = i2cRead() << 16 | i2cRead();  //Read the value of the slider
  return value;
}

int requestToggleState(int region){
  char command[] = {254, 171, region}; //Send the Toggle State command parameters
  i2cWrite(command, sizeof(command));
}

int readToggleState(){
  int returnLength = i2cRead() << 16 | i2cRead();  //Read message length  
  int result = i2cRead(); //Read the eStatus result of the command
  int toggleState = i2cRead();   //Read the value of the slider
  return toggleState;
}

void updateLabel(byte index, byte format, char* message, byte messageLength){
  char command[] = { 254, 17, index, format }; //Send the label update command parameters
  i2cWrite(command, sizeof(command));
  i2cWrite(message, messageLength); //Send the ASCII message to the GTT
  char terminator[] = { 0 };  
  i2cWrite(terminator, 1);  //Terminate ASCII string
}

void updateBargraphValue(int index, int value){
  byte val1 = value >>8;  //Separate the bargraph value into 2 8 bit values
  byte val2 = value & 0xFF;
  char command[] = { 254, 105, index, val1, val2};  //Send the update bargraph command parameters
  i2cWrite(command, sizeof(command));
}

//Reading returns
int ReadEvent(){
  int returnLength = i2cRead() << 16 | i2cRead();  //Read message length
  int event = i2cRead();  //Read event type
  return event;
}
int ReadRegion() { //Determine which region has been pressed  
  int region = i2cRead();  //Read region type
  return region;
}

//I2C communication commands
byte i2cRead() {
  do {
    Wire.requestFrom(gttAddress, 1);
  } while (Wire.available() < 1);
  return (Wire.read());
}

void i2cWrite(char* data, byte data_length) {
  Wire.beginTransmission(gttAddress);
  for (int i = 0; i < data_length; i++) {
    Wire.write(data[i]);
  }
  Wire.endTransmission();
}
